/**
 * Enum representing the possible sides of an action.
 */
export enum ActionSide {
    Buy,
    Sell,
}

/**
 * Represents an action taken on a stock, such as a buy or sell order.
 */
export class Action {
    /**
     * The symbol of the asset being traded.
     */
    readonly symbol: string;

    /**
     * The quantity of the asset being traded.
     */
    readonly quantity: number;
    
    /**
     * The side of the action, either "buy" or "sell".
     */
    readonly side: ActionSide;

    /**
     * The price per share of the asset.
     */
    readonly pricePerShare: number;

    /**
     * Creates an instance of an Action.
     * @constructor
     * @param {string} symbol - The symbol of the asset being traded.
     * @param {number} quantity - The quantity of the asset being traded.
     * @param {ActionSide} side - The side of the trade (buy or sell).
     * @param {number} pricePerShare - The price per share of the asset being traded.
     */
    constructor(symbol: string, quantity: number, side: ActionSide, pricePerShare: number) {
        this.symbol = symbol;
        this.quantity = quantity;
        this.side = side;
        this.pricePerShare = pricePerShare;
    }
}

export enum ActionDateType {
    On,
    After,
    Before,
}

export class ActionDateOptions {
    readonly date: Date;
    readonly dateType: ActionDateType;

    constructor(date: Date, dateType: ActionDateType) {
        this.date = date;
        this.dateType = dateType;
    }
}

export class ActionFetchOptions {
    readonly pageSize?: number;
    readonly dateOptions?: ActionDateOptions;
}

export interface ActionFetchResponse {
    readonly actions: Action[];

    readonly fetchNextPage?: () => Promise<ActionFetchResponse>;
}

/**
 * An interface representing an object that provides actions.
 */
export interface ActionProvider {
    /**
     * Fetches actions based on the provided options.
     * @param options - The options to use when fetching actions.
     * @returns A promise that resolves to an object containing the fetched actions.
     */
    readonly fetchActions: (options: ActionFetchOptions) => Promise<ActionFetchResponse>;
}